      SUBROUTINE SIGMA ( N, X0, H, EPS, DX,
     1                   NTRAJ, ISEGBR, KPBR0, INKPBR,
     2                   NPMIN, NPMAX0, INPMAX,
     3                   NSUC, NTRIAL, TOLREL, TOLABS, XRMIN, XRMAX,
     4                   KPASCA, IRAND, INHP, IPRINT,
     5                   XMIN, FMIN, NFEV, IOUT )
C
C  THE SUBROUTINE  SIGMA  IS THE PRINCIPAL SUBROUTINE OF THE PACKAGE
C  SIGMA, WHICH ATTEMPTS TO FIND A GLOBAL MINIMIZER OF A REAL VALUED
C  FUNCTION  F(X) = F(X1,...,XN)  OF  N  REAL VARIABLES  X1,...,XN.
C  THE ALGORITHM AND THE PACKAGE ARE DESCRIBED IN DETAIL IN THE TWO
C  PAPERS PUBLISHED IN THE SAME ISSUE OF THE A.C.M. TRANSACTIONS ON
C  MATHEMATICAL SOFTWARE, BOTH BY
C     F. ALUFFI-PENTINI, V. PARISI, F. ZIRILLI.
C  (1)  A GLOBAL MINIMIZATION ALGORITHM USING STOCHASTIC DIFFERENTIAL
C       EQUATIONS
C  (2)  ALGORITHM SIGMA, A STOCHASTIC-INTEGRATION GLOBAL MINIMIZATION
C       ALGORITHM.
C  THE SOFTWARE IMPLEMENTATION AND ITS USAGE ARE DESCRIBED IN (2).
C
C  METHOD
C
C  A GLOBAL MINIMIZER OF  F(X)  IS SOUGHT BY MONITORING THE VALUES OF
C  F  ALONG TRAJECTORIES GENERATED BY A SUITABLE (STOCHASTIC) DISCRE-
C  TIZATION OF A FIRST-ORDER STOCHASTIC DIFFERENTIAL EQUATION INSPIRED
C  BY STATISTICAL MECHANICS. STARTING FROM AN INITIAL POINT  X0 ,
C  X  IS UPDATED BY THE (STOCHASTIC) DISCRETIZATION STEP
C       X = X + DX1 + DX2
C  WHERE  DX1 = - H * GAM           (FIRST HALF-STEP)
C         DX2 = EPS * SQRT(H) * U   (SECOND HALF-STEP)
C  AND  H  IS THE TIME-INTEGRATION STEPLENGTH,
C  GAM/N  IS COMPUTED AS A FINITE-DIFFERENCE APPROXIMATION TO THE
C     DIRECTIONAL DERIVATIVE OF  F  ALONG AN ISOTROPICALLY RANDOM
C     DIRECTION,
C  EPS  IS A POSITIVE 'NOISE' COEFFICIENT, AND
C  U  IS A RANDOM SAMPLE FROM AN N-DIMENSIONAL GAUSSIAN DISTRIBUTION.
C  WE CONSIDER THE SIMULTANEOUS EVOLUTION OF A GIVEN FIXED NUMBER
C  NTRAJ  OF TRAJECTORIES DURING AN OBSERVATION PERIOD IN WHICH FOR
C  EACH TRAJECTORY  EPS  IS FIXED WHILE  H  AND THE SPATIAL DISCRETI-
C  ZATION INCREMENT  DX  FOR COMPUTING  GAM  ARE AUTOMATICALLY
C  ADJUSTED BY THE ALGORITHM.
C  AFTER EVERY OBSERVATION PERIOD ONE OF THE TRAJECTORIES IS DISCARDED,
C  ALL OTHER TRAJECTORIES CONTINUE UNPERTURBED, AND ONE OF THEM IS SE-
C  LECTED FOR BRANCHING, I.E. GENERATING A SECOND PERTURBED CONTI-
C  NUATION, WITH DIFFERENT STARTING  EPS  AND  DX  (AND THE SAME
C  'PAST HISTORY' OF THE FIRST).
C  THE SET OF SIMULTANEOUS TRAJECTORIES IS CONSIDERED A SINGLE TRIAL,
C  AND THE COMPLETE ALGORITHM IS A SET OF REPEATED TRIALS.
C  A TRIAL IS STOPPED, AT THE END OF AN OBSERVATION PERIOD, AND AFTER
C  HAVING DISCARDED THE WORST TRAJECTORY, IF ALL THE FINAL VALUES OF
C  F  FOR THE REMAINING TRAJECTORIES ARE EQUAL (WITHIN NUMERICAL TO-
C  LERANCES, AND POSSIBLY AT DIFFERENT POINTS  X) TO THEIR MINIMUM
C  VALUE  FTFMIN ('UNIFORM STOP AT THE LEVEL  FTFMIN ').
C  A UNIFORM STOP IS CONSIDERED SUCCESSFUL ONLY IF THE FINAL VALUE
C  FTFMIN  IS (NUMERICALLY) EQUAL TO THE CURRENT BEST MINIMUM  FOPT
C  FOUND SO FAR FROM ALGORITHM START.
C  A TRIAL IS ALSO ANYWAY STOPPED (UNSUCCESSFULLY) IF A GIVEN MAXIMUM
C  NUMBER  NPMAX  OF OBSERVATION PERIODS HAS ELAPSED.
C  TRIALS ARE REPEATED WITH DIFFERENT OPERATING CONDITIONS (INITIAL
C  POINT, MAX TRIAL LENGTH  NPMAX , SEED OF NOISE GENERATOR, POLICY
C  FOR CHOOSING THE STARTING  EPS  FOR THE PERTURBED CONTINUATION,
C  AND TRIAL-START VALUE OF  EPS ).
C  THE OUTCOMES OF ALL THE COMPLETED TRIALS ARE SUMMARIZED BY
C  FTFOPT (THE BEST CURRENT MIMNIMUM VALUE FOUND SO FAR FOR
C  FTFMIN ) AND BY THE CURRENT COUNT  KSUC  OF SUCCESSFUL TRIALS
C  (WHICH IS ZERO AT ALGORITHM START, AND IS UPDATED AT EVERY
C  SUCCESSFULLY-STOPPING TRIALS).
C  THE ALGORITHM IS STOPPED, AT THE END OF A TRIAL, IF A REQUESTED
C  COUNT  NSUC  OF SUCCESSFUL TRIALS HAS BEEN REACHED,
C  OR ANYWAY IF A GIVEN MAXIMUM NUMBER  NTRIAL  OF TRIALS HAS BEEN
C  REACHED.
C  SUCCESS IS CLAIMED IF THE CURRENT COUNT  KSUC  OF SUCCESSFUL TRIALS
C  IS AT LEAST ONE, AND IF SUCH TRIALS ARE CURRENTLY VALID.
C
C  CALL STATEMENT
C
C  THE CALL STATEMENT IS
C     CALL SIGMA ( N, X0, H, EPS, DX,
C                  NTRAJ, ISEGBR, KPBR0, INKPBR,
C                  NPMIN, NPMAX0, INPMAX,
C                  NSUC, NTRIAL, TOLREL, TOLABS, XRMIN, XRMAX,
C                  KPASCA, IRAND, INHP, IPRINT,
C                  XMIN, FMIN, NFEV, IOUT )
C
C  CALL PARAMETERS
C
C  INPUT PARAMETERS ARE THOSE IN LINES 1,3,4,5 OF THE CALL STATEMENT,
C  INPUT-OUTPUT PARAMETERS ARE THOSE IN LINE 2,
C  OUTPUT PARAMETERS ARE THOSE IN LINE 6.
C  NOTE THAT A NUMBER OF OTHER (INTERNAL) PARAMETERS CAN BE OBTAINED
C  BY MEANS OF THE USER-SUPPLIED OUTPUT SUBROUTINES  PTSEG, PTRIAL,
C  AND  PTKSUC, WHICH ARE DESCRIBED BELOW.
C
C  DESCRIPTION OF THE CALL PARAMETERS
C
C  N        IS THE PROBLEM DIMENSION (NUMBER OF VARIABLES)
C  X0       IS AN  N-VECTOR CONTAINING THE INITIAL VALUES OF THE
C          X-VARIABLES
C  H       IS THE INITIAL VALUE OF THE TIME-INTEGRATION STEPLENGTH.
C  EPS     IS THE INITIAL VALUE OF THE NOISE COEFFICIENT.
C  DX      IS THE INITIAL VALUE OF THE MAGNITUDE OF THE DISCRETIZATION
C          INCREMENT FOR COMPUTING THE FINITE-DIFFERENCE DERIVATIVES.
C  NTRAJ   IS THE NUMBER OF SIMULTANEOUS TRAJECTORIES.
C          (NOTE HOWEVER THAT IF THE INPUT VALUE IS ZERO,  NTRAJ  IS
C          SET TO A DEFAULT VALUE (NTRAJ = 7), AND IF THE INPUT VALUE
C          IS OTHERWISE OUTSIDE THE INTERVAL (3,20) NTRAJ IS SET TO
C          THE NEAREST EXTREME VALUE).
C  ISEGBR, KPBR0, INKPBR  DETERMINE, AT THE END OF AN OBSERVATION
C          PERIOD, WHICH ONE OF THE SIMULTANEOUS TRAJECTORIES
C          IS TO BE BRANCHED, AS FOLLOWS.
C          BRANCHING IS NORMALLY PERFORMED ON THE TRAJECTORY WHICH
C          OCCUPIES THE PLACE  ISEGBR  IN THE TRAJECTORY SELECTION OR-
C          DERING, EXCEPT AT (THE END OF) EXCEPTIONAL OBSERVATION
C          PERIODS, WHERE THE FIRST TRAJECTORY IN THE ORDERING IS
C          BRANCHED. EXCEPTIONAL BRANCHING OCCURS AT THE OBSERVATION
C          PERIODS NUMBERED  KP = KPBR0 + J*INKPBR, (J = 1,2,3,...).
C          THEREFORE  ISEGBR  SELECTS THE LEVEL (IN THE ORDERING) AT
C          WHICH NORMAL BRANCHING OCCURS, WHILE  KPBR0  AND  INKPBR
C          SELECT THE FIRST OCCURRENCE AND THE REPETITION FREQUENCY
C          OF THE EXCEPTIONAL OBSERVATION PERIODS.
C          (NOTE HOWEVER THAT IF ONE OF THE INPUT VALUES IS ZERO,
C          THE CORRESPONDING VARIABLE IS SET TO A DEFAULT VALUE
C          ISEGBR = INT((1+NTRAJ)/2), INKPBR = 10, KPBR0 = 3.
C          IF THE INPUT VALUE FOR  ISEGBR  IS OTHERWISE OUTSIDE THE
C          INTERVAL  (1,NTRAJ),  ISEGBR  IS SET TO THE NEAREST
C          EXTREME VALUE, AND IF KPBR0  HAS A VALUE NOT INSIDE THE
C          INTERVAL  (1,INKPBR), IT IS ASSIGNED THE SAME VALUE
C          MODULO  INKPBR).
C  NPMIN   IS THE MINIMUM DURATION OF A TRIAL, I.E. THE MINIMUM
C          NUMBER OF OBSERVATION PERIODS THAT SHOULD ELAPSE BEFORE
C          STARTING TO CHECK THE TRIAL STOPPING CRITERIA.
C  NPMAX0  IS THE MAXIMUM DURATION OF THE FIRST TRIAL, I.E. THE
C          VALUE, FOR THE FIRST TRIAL, OF MAXIMUM ACCEPTABLE
C          NUMBER  NPMAX  OF OBSERVATION PERIODS IN A TRIAL.
C  INPMAX  IS THE INCREMENT FOR  NPMAX , WHEN  NPMAX  IS VARIED
C          FROM ONE TRIAL TO THE FOLLOWING ONE.
C  NSUC    IS THE REQUESTED NUMBER OF SUCCESSFUL TRIALS
C          AFTER WHICH THE COMPUTATION IS STOPPED.
C  TOLREL  AND  TOLABS  ARE THE RELATIVE AND ABSOLUTE TOLERANCES
C          FOR STOPPING A SINGLE TRIAL.
C  XRMIN,  XRMAX  ARE N-VECTORS DEFINING THE ADMISIBLE REGION FOR
C          THE X-VALUES, WITHIN WHICH THE FUNCTION VALUES CAN BE
C          SAFELY COMPUTED.
C  KPASCA  IS THE MINIMUM NUMBER OF TRAJECTORY SEGMENTS THAT SHOULD
C          ELAPSE BEFORE THE RESCALING PROCEDURES ARE ACTIVATED.
C  IRAND   IS A CONTROL INDEX FOR THE INITIALIZATION OF THE RANDOM
C          NUMBER GENERATOR.
C          IRAND.GT.0    THE GENERATOR IS INITIALIZED, BEFORE STAR-
C                        TING THE TRIAL  KT, WITH SEED  IRAND+KT-1
C          IRAND.LE.0    THE GENERATOR IS INITIALIZED (WITH SEED 0)
C                        ONLY AT THE FIRST CALL OF SIGMA
C  INHP    IS A CONTROL INDEX FOR SELECTING THE NUMBER  NHP  OF TIME-
C          INTEGRATION STEPS FOR OBSERVATION PERIOD  KP (DURATION OF
C          TRIAL  KP) AS FOLLOWS (LOG IS BASE 2)
C          INHP=1  NHP = 1 + INT(LOG(KP))      ('SHORT' DURATION)
C          INHP=2  NHP = INT(SQRT(KP))         ('MEDIUM' DURATION)
C          INHP=3  NHP = KP                    ('LONG' DURATION)
C  IPRINT  IS AN INDEX USED TO CONTROL THE AMOUNT OF PRINTED OUTPUT
C          BY CONTROLLING THE CALLS TO THE USER-SUPPLIED OUTPUT SUB-
C          ROUTINES  PTSEG  (END-OF-SEGMENT OUTPUT),  PTRIAL  (END-
C          OF-TRIAL OUTPUT), AND  PTKSUC (END-OF-TRIAL OUTPUT RELATED
C          TO THE COUNT OF SUCCESSFUL TRIALS), WHICH ARE DESCRIBED
C          BELOW.
C          IPRINT.LT.0   NO CALL TO THE OUTPUT SUBROUTINES
C          IPRINT.EQ.0   CALL ONLY  PTRIAL  AND  PTKSUC
C          IPRINT.GT.0   CALL ALL OUTPUT SUBROUTINES.
C  XMIN    IS AN N-VECTOR CONTAINING THE COORDINATES OF THE POINT
C          (OR POSSIBLY ONE OF THE POINTS) WHERE THE FINAL VALUE  FMIN
C          OF  FOPT  WAS FOUND.
C  FMIN    IS THE FINAL VALUE OF THE BEST CURRENT MINIMUM FUNCTION
C          VALUE  FOPT.
C  NFEV    IS THE TOTAL NUMBER OF FUNCTION EVALUATION (INCLUDING
C          THOSE USED FOR THE COMPUTATION OF DERIVATIVES, AND FOR
C          THE REJECTED TIME-INTEGRATION STEPS).
C  IOUT    IS THE INDICATOR OF THE STOPPING CONDITIONS, AS FOLLOWS
C          IF  IOUT = -99  A FATAL ERROR WAS DETECTED WHEN PERFOR-
C             MING SOME PRELIMINARY CHECKING OF THE INPUT DATA, AND
C             THE ALGORITHM WAS NOT EVEN STARTED
C          OTHERWISE THE ALGORITHM WAS STARTED, AND THE MEANING OF
C          IOUT  IS AS FOLLOWS
C          IF  IOUT.EQ.0  NO TRIAL HAD A UNIFORM STOP.
C          IF  IOUT.NE.0  THE SIGN AND THE MAGNITUDE OF  IOUT  HAVE
C          THE FOLLOWING MEANING
C          A) THE SIGN OF  IOUT  INDICATES IF THE BEST UNIFORM STOP
C          (I.E. THE ONE IN WHICH  FTFOPT  WAS OBTAINED)
C          IS TO BE CONSIDERED SUCCESSFUL (IOUT.GT.0) OR UNSUCCESSFUL
C          B) THE MAGNITUDE OF  IOUT  INDICATES WHICH ONE OF THE
C          NUMERICAL EQUALITY CRITERIA WAS SATISFIED IN THE BEST
C          UNIFORM STOP
C          IABS(IOUT).EQ.1  RELATIVE DIFFERENCE CRITERION
C          IABS(IOUT).EQ.2  ABSOLUTE DIFFERENCE CRITERION
C          IABS(IOUT).EQ.3  BOTH CRITERIA
C          (IOUT  IS THE FINAL VALUE OF THE INTERNAL PARAMETER  ISTOPT
C          (AN OUTPUT INDICATOR OF THE USER-SUPPLIED SUBROUTINE
C          PTRIAL )).
C          SUCCESS IS CLAIMED BY THE ALGORITHM IF  IOUT .GT. 0,
C          I.E. IF AT LEAST ONE OF THE TRIALS STOPPED SUCCESSFULLY
C          (I.E. WITH A POSITIVE VALUE OF THE TRIAL STOP INDICATOR
C          ISTOP ), AND THE SUCCESSFUL TRIALS (AS COUNTED BY  KSUC )
C          ARE CURRENTLY VALID.
C
C
C  USER-SUPPLIED SUPROGRAMS
C
C  THE USER MUST PROVIDE THE FUNCTION  FUNCT  TO COMPUTE  F(X),
C  AND THE THREE OUTPUT SUBROUTINE  PTSEG, PTRIAL, PTKSUC .
C  THE CALLS TO THE OUTPUT SUBROUTINES ARE CONTROLLED BY  IPRINT
C  (INPUT PARAMETER TO  SIGMA).
C  A USER NOT INTERESTED IN USING ANY ONE OF THE OUTPUT SUBROUTINES
C  CAN SIMPLY PUT  IPRINT = -1.
C  SAMPLE VERSIONS OF THE OUTPUT SUBROUTINES ARE PROVIDED
C  WITH THE PACKAGE FOR THE BENEFIT OF THE AVERAGE USER.
C  IN THE FOLLOWING DESCRIPTION ALL NON-INTEGER ARGUMENTS ARE
C  DOUBLE PRECISION (INTEGER ARGUMENTS ARE INDICATED BY MEANS OF THE
C  FORTRAN IMPLICIT TYPE DEFINITION CONVENTION).
C
C     THE FUNCTION  FUNCT
C
C     FUNCT  MUST RETURN AS ITS VALUE THE VALUE AT  X  OF THE FUNCTION
C     TO BE MINIMIZED
C     THE DEFINITION STATEMENT IS
C        DOUBLE PRECISION FUNCTION  FUNCT (N, X)
C     WHERE
C     N   IS THE (INPUT) DIMENSION OF THE PROBLEM.
C     X   IS THE (INPUT) N-VECTOR CONTAINING THE COORDINATES OF THE
C         POINT  X  WHERE THE FUNCTION IS TO BE COMPUTED.
C
C     THE SUBROUTINE  PTSEG
C
C     PTSEG  IS CALLED (IF  IPRINT.GT.0) AT THE END OF EVERY OBSER-
C     VATION PERIOD.
C     THE DEFINITION STATEMENT IS
C         SUBROUTINE  PTSEG ( N, XPFMIN, FPFMIN, FPFMAX,
C                                         KP, NFEV, IPRINT )
C     WHERE
C     N   IS THE (INPUT) DIMENSION OF THE PROBLEM
C     FPFMIN, FPFMAX  ARE RESPECTIVELY THE MINIMUM AND THE MAXIMUM
C        AMONG THE VALUES OF  F(X) OBTAINED AT THE FINAL POINTS OF
C        THE TRAJECTORY SEGMENTS OF THE (JUST ELAPSED) OBSERVATION
C        PERIOD  KP.
C     XPFMIN   IS AN N-VECTOR CONTAINING THE COORDINATES OF THE
C        (FINAL) POINT (OR POSSIBLY ONE OF THE POINTS) WHERE
C        FPFMIN WAS OBTAINED.
C     KP   IS THE TOTAL NUMBER OF ELAPSED OBSERVATION PERIODS IN
C        THE CURRENT TRIAL.
C     NFEV   IS THE TOTAL NUMBER OF FUNCTION EVALUATIONS PERFORMED
C        FROM ALGORITHM START.
C
C     THE SUBROUTINE  PTRIAL
C
C     PTRIAL  IS CALLED (IF IPRINT.GE.0) AT THE END OF EVERY TRIAL.
C     THE DEFINITION STATEMENT IS
C         SUBROUTINE PTRIAL ( N, XOPT, FOPT,
C                             FTFMIN, FTFMAX, FTFOPT,
C                             ISTOP, ISTOPT, NFEV, KP, IPRINT )
C     WHERE
C     N   IS THE (INPUT) DIMENSION OF THE PROBLEM.
C     XOPT   IS AN N-VECTOR CONTINING THE COORDINATES OF THE
C        POINT (OR POSSIBLY ONE OF THE POINTS) WHERE THE CURRENT
C        MINIMUM  FOPT  WAS OBTAINED.
C     FOPT   IS THE CURRENT BEST MINIMUM VALUE FOUND FOR  F  FROM
C        ALGORITHM START ( FOPT  IS UPDATED WHENEVER A FUNCTION
C        VALUE IS COMPUTED).
C     FTFMIN, FTFMAX   ARE RESPECTIVELY THE MINIMUM AND THE MAXIMUM
C        AMONG THE VALUES OF  F(X)  OBTAINED AT THE FINAL POINTS OF
C        THE LAST TRAJECTORY SEGMENTS OF THE CURRENT TRIAL.
C     FTFOPT   IS THE CURRENT MINIMUM VALUE OF  FTFMIN  AMONG THE
C        TRIALS WHICH DID NOT STOP FOR REACHING THE MAXIMUM ALLOWED
C        NUMBER OF SEGMENTS (STOPPING INDICATOR  ISTOP = 0, SEE
C        BELOW).  FTFOPT  IS USED BY  SIGMA  TO COMPUTE  KSUC  (INPUT
C        PARAMETER TO THE OUTPUT SUBROUTINE  PTKSUK , SEE BELOW).
C     KP   IS THE TOTAL NUMBER OF ELAPSED OBSERVATION PERIODS IN
C        THE CURRENT TRIAL.
C     NFEV   IS THE TOTAL NUMBER OF FUNCTION EVALUATIONS PERFORMED
C        FROM ALGORITHM START.
C     ISTOP   IS THE INDICATOR OF THE STOPPING CONDITION OF THE TRIAL,
C        AS FOLLOWS
C        ISTOP = 0
C           THE MAXIMUM NUMBER  NPMAX  OF OBSERVATION PERIODS HAS
C           BEEN REACHED.
C        ISTOP.NE.0
C           ALL THE END-OF-SEGMENT VALUES OF  F(X) , (EXCEPT FOR THE
C           JUST DISCARDED SEGMENT) ARE CLOSE ENOUGH TO THEIR COMMON
C           MINIMUM VALUE  FPFMIN , WITH RESPECT TO AN ABSOLUTE OR
C           RELATIVE DIFFERENCE CRITERION, TO BE CONSIDERED NUMERI-
C           CALLY EQUAL.
C           THE ABSOLUTE VALUE AND THE SIGN OF  ISTOP  HAVE THE
C           FOLLOWING MEANING.
C              THE ABSOLUTE VALUE INDICATES WHICH DIFFERENCE
C                 CRITERION WAS SATISFIED
C                 1   RELATIVE DIFFERENCE CRITERION SATISFIED
C                 2   ABSOLUTE DIFFERENCE CRITERION SATISFIED
C                 3   BOTH CRITERIA SATISFIED
C              THE SIGN OF  ISTOP  INDICATES THE RELATIONSHIP BETWEEN
C                 THE END-OF-TRIAL VALUE  FPFMIN  AND THE CURRENT
C                 BEST MINIMUM VALUE  FOPT  (WHICH IS UPDATED WHEN-
C                 EVER A FUNCTION  VALUE IS COMPUTED
C                 ISTOP.GT.0
C                    FPFMIN  IS NUMERICALLY EQUAL (W.R.T. AT LEAST
C                    ONE OF THE ABOVE  DIFFERENCE CRITERIA) TO  FOPT
C                 ISTOP.LT.0
C                    FPFMIN  IS NOT EVEN NUMERICALLY EQUAL TO  FOPT
C                    (AND THEREFORE CANNOT BE CONSIDERED AS AN
C                    ACCEPTABLE GLOBAL MINIMUM).
C     ISTOPT   IS THE VALUE OF THE TRIAL STOPPING INDICATOR  ISTOP
C        CORRESPONDING TO THE (CURRENT OR PAST) TRIAL WHERE  FTFOPT
C        WAS OBTAINED, WITH THE SIGN WHICH IS UPDATED ACCORDING TO
C        THE COMPARISON BETWEEN  FTFOPT  AND THE PRESENT VALUE OF
C        FOPT , AS DESCRIBED ABOVE.
C        THE FINAL VALUE OF  ISTOP IS RETURNED BY  SIGMA  AS THE VALUE
C        OF THE OUTPUT INDICATOR IOUT OF THE ALGORITHM STOPPING CON-
C        DITIONS (WHENEVER THE ALGORITHM WAS STARTED, IOUT.NE.-99,
C        SEE ABOVE).
C
C     THE SUBROUTINE  PTKSUC
C
C     PTKSUC  IS CALLED ONLY AT THE END OF EVERY SUCCESSFUL TRIAL
C     SUCH THAT AN INCREMENT OCCURRED IN THE VALUE OF THE
C     CURRENT MAXIMUM  MSUC  AMONG ALL THE VALUES OF  KSUC  FROM
C     ALGORITHM START. A CALL TO  PTKSUC  THEREFORE PROVIDES
C     THE USER WITH THE OPERATIONALLY INTERESTING INFORMATION THAT
C     ALGORITHM STOP WOULD HAVE TAKEN PLACE, IF  NSUC  (INPUT
C     PARAMETER TO  SIGMA ) HAD BEEN GIVEN A LOWER VALUE, EQUAL TO
C     THE CURRENT  KSUC .
C     PTKSUC  IS CALLED ONLY IF  IPRINT.GE.0  AND  KSUC.LT.NSUC  .
C     THE DEFINITION STATEMENT IS
C         SUBROUTINE  PTKSUC ( KSUC )
C     WHERE  KSUC  IS THE CURRENT COUNT OF SUCCESSFUL TRIALS
C     (1 .LE. KSUC .LE. NSUC)
C
      DOUBLE PRECISION X0,H,EPS,DX,TOLREL,TOLABS
      DOUBLE PRECISION XRMIN,XRMAX,XMIN,FMIN
      DOUBLE PRECISION EPSAG,EPSAP,EPSC
      DOUBLE PRECISION EPSMAX,EPSR,F,FOPT,FTFMAX
      DOUBLE PRECISION FTFMIN,FTFOPT
C
      DOUBLE PRECISION X,HC,DXC,VMVT,EPSCO,VMCOR,VCOR
      DOUBLE PRECISION XRMIC,XRMAC,XOPT,FOPTC
C
      DOUBLE PRECISION DIST,BIAS,GRAGRA,GRA
C
      DIMENSION X0(N),XMIN(N),XRMIN(N),XRMAX(N)
C
      COMMON /DINCOM/ X(100,20),HC(20),DXC(20),VMVT(20,19),EPSCO(20),
     1    VMCOR(20),VCOR(20),XRMIC(100),XRMAC(100),XOPT(100),FOPTC,
     2    IE(20),ISVT(20,19),KGEN,KTIM,NDIM,NTRAJC,NTRAJR,
     3    ISEGBC,INKPBC,KPBR0C,NCF,IFEPC,INHPC
C
      COMMON /SCALE/ DIST(10,10,20),BIAS(10,20),GRAGRA(10,10,20),
     1    GRA(10,20),NGRA(20),LSCA,IDSCA,NX,NORD
C
C  DATA FOR THE VARIATION OF NOISE COEFFICIENT
C
      DATA EPSR/1.D4/,EPSAP/10.D0/,EPSAG/1.D3/
      DATA EPSMAX/1.D15/
C
      IFEP = 1
C
C  INITIALIZE COMMON AREA  /DINCOM/
C
      CALL INIT(N,X0,H,EPS,DX,IRAND,F,
     1    NTRAJ,ISEGBR,INKPBR,KPBR0,INHP,IFEP,XRMIN,XRMAX,IOUT)
C
C  CHECK PARAMETER VALUES
C
      IF(NPMIN.LE.0.OR.NPMAX0.LT.0.OR.INPMAX.LE.0.OR.
     1    NSUC.LE.0.OR.NTRIAL.LE.0)IOUT = -99
      IF (IOUT.EQ.(-99))RETURN
C
C  INITIALIZE VARIABLES
C
      EPSC = EPS
      NPMAX = NPMAX0
      ISTOPT = 0
      ISTOP = 0
      ICCOM = (NTRIAL+NTRIAL+4)/5
      NFEV = 0
      NTES = NSUC
      ICTS = NSUC-1
      FTFOPT = F
C
C  START SERIES OF TRIALS
C
      DO 30 IC = 1,NTRIAL
C
C  SET INITIALIZATION INDEX FOR NOISE GENERATOR
C
         IS = 0
         IF(IRAND.GT.0)IS = IRAND+IC-1
C
C  INITIALIZE TRIAL
C
         IF(IC.GT.1.AND.IC.LE.ICCOM)CALL REINIT(N,X0,EPSC,IS,F,IFEP)
         IF(IC.GT.ICCOM)CALL REINIT(N,XMIN,EPSC,IS,FOPT,IFEP)
C
         FTFMIN = F
         FTFMAX = F
         NFEV = NFEV+1
C
C  PRINT INITIAL CONDITIONS OF TRIAL
C
         IF(IPRINT.GT.0)CALL PTSEG(N,X0,FTFMIN,FTFMAX,0,NFEV)
C
C
C  DEACTIVATE SCALING
C
         IF(KPASCA.GT.NPMAX.OR.N.LE.1)CALL NOSCA
C
C  INITIALIZE COMMON AREA  /SCALE/
C
         IF(KPASCA.LE.NPMAX.AND.N.GT.1)CALL INISCA(N,NTRAJ)
C
C  PERFORM A TRIAL
C
         CALL TRIAL(N,NPMIN,NPMAX,KPASCA,TOLREL,TOLABS,
     1       IPRINT,XMIN,FTFMIN,
     1       FTFMAX,NFEV,KP,ISTOP)
C
C
C     EVALUATE PAST TRIAL AND PREPARE NEXT TRIAL
C
C
C  SET TRIAL DURATION
C
         IF(ISTOP.EQ.0)NPMAX = NPMAX+INPMAX
C
C  RESET CURRENT NUMBER OF SUCCESSES REQUIRED BEFORE STOPPING
C  COMPUTE INDICATOR OF TRIAL STOPPING CONDITIONS
C  UPDATE BEST CURRENT VALUES OF TRIAL STOPPING INDICATOR AND
C  OF FUNCTION  F(X)  AT TRIAL STOP
C
         IF((FTFMIN.GT.FTFOPT.OR.ISTOP.EQ.0).AND.ISTOPT.NE.0)GO TO 10
         IF(ITOLCH(FTFOPT,FTFMIN,TOLREL,TOLABS).EQ.0)NTES = NSUC
C
         FTFOPT = FTFMIN
         ISTOPT = ISTOP
  10     CONTINUE
         ISTOPT = IABS(ISTOPT)
         CALL RCLOPT(N,XMIN,FOPT)
         IF(ITOLCH(FTFOPT,FOPT,TOLREL,TOLABS).EQ.0)ISTOPT = -ISTOPT
         IF(ITOLCH(FTFMIN,FOPT,TOLREL,TOLABS).EQ.0)ISTOP = -ISTOP
C
C  END-OF-TRIAL PRINT OUT
C
         IF(IPRINT.GE.0)
     1      CALL PTRIAL ( N, XOPT, FOPT,
     2                    FTFMIN, FTFMAX, FTFOPT,
     3                    ISTOP, ISTOPT, NFEV, KP, IPRINT )
C
C  UPDATE INITIAL VALUE OF NOISE COEFFICIENT FOR NEXT TRIAL
C
         IF (ISTOP.EQ.0)EPSC = EPSC/EPSR
         IF(ISTOP.GT.0)EPSC = EPSC*EPSAG
         IF(ISTOP.LT.0.AND.IC.LE.ICCOM)EPSC = EPSC*EPSAP
         IF(ISTOP.LT.0.AND.IC.GT.ICCOM)EPSC = EPSC/EPSR
C
C  UPDATE OPERATING CONDITIONS FOR SELECTING (IN THE NEXT TRIAL)
C  THE STARTING VALUE OF THE NOISE COEFFICIENT OF THE NEW TRAJECTORY
C  AFTER BRANCHING
C
         IF (ISTOP.EQ.0)IFEP = 1
         IF (ISTOP.NE.0)IFEP = 2
C
C  UPDATE, PRINT, AND CHECK TRIAL STOPPING CONDITIONS
C
         IF(ISTOP.GT.0.AND.ISTOPT.GT.0)NTES = NTES-1
         IF(NTES.LE.0.OR.ISTOPT.LE.0.OR.ISTOP.LE.0.OR.NTES.GT.ICTS.OR.
     1       IPRINT.LT.0) GO TO 20
         KSUC = NSUC-ICTS
         CALL PTKSUC(KSUC)
         ICTS = NTES-1
  20     CONTINUE
         IF(NTES.LE.0.AND.ISTOPT.GT.0.AND.ISTOP.GT.0)GO TO 40
C
C  CONSTRAIN NOISE COEFFICIENT WITHIN BOUNDS
C
         EPSC = DMIN1(EPSC,EPSMAX)
         IF(EPSC.LE.0.D0)EPSC = 1.D0
  30     CONTINUE
C  END OF SERIES OF TRIALS
C
  40  CONTINUE
C
C  INDICATOR OF STOPPING CONDITIONS
C
      IOUT = ISTOPT
      FMIN = FOPT
C
      RETURN
C
      END
